import React, { Fragment, useState, useEffect, useMemo, useRef } from 'react';
import { useNavigate } from 'umi';
import {
  Card,
  Table,
  Empty,
  Button,
  Modal,
  message,
  Form,
  DatePicker,
  Row,
  Col,
} from 'antd';
import { PageContainer } from '@ant-design/pro-components';
const { RangePicker } = DatePicker;
import { useRequest, useModel } from '@umijs/max';
import { queryProfileList } from '@/services/boom';
import dayjs from 'dayjs';
import { queryContractCheck, queryGetContractList } from '@/services/contract';
import ContractModal, {
  Status,
  StatusText,
  Type,
} from '../ContractManager/component/Modal';
function profile(props) {
  const { submitting, params, dispatch } = props;
  const { user } = useModel('userInfo');
  const [tabActive, setTabActive] = useState('1');
  const approveFormRef = useRef();
  const applyFormRef = useRef();
  let navigate = useNavigate();
  const [detail, setDetail] = useState({});
  const [conVisible, setConVisible] = useState(false);

  const TYPE = {
    Contract: 1,
    OA: 2,
  };

  const { data: OAData, loading: OALoading } = useRequest(queryProfileList, {
    // manual: true,
    formatResult: (res) => {
      return res.data?.list?.map((item) => {
        return {
          ...item,
          CName: item.AuthorInfo.CName,
          table_desc: [item.table_desc],
          statusText: item.status,
          type: TYPE.OA,
        };
      });
    },
    onSuccess: (data) => {
      console.log(data);
    },
  });
  const contractResult = (res) => {
    let data = res.data?.list;
    return data?.map((item) => {
      return {
        ...item,
        table_name: `${user.CName}提交的合同审批`,
        table_desc: [
          `合同名称：${item.name}`,
          `合同编号：${item.code}`,
          `合同金额：${item.amount}万元`,
        ],
        CName: user.CName,
        create_time: item.cancel_on || item.created_on,
        type: TYPE.Contract,
        statusText: StatusText[item.status],
      };
    });
  };

  //合同管理相关数据
  //请求我的申请列表
  const {
    data: ApplyData,
    run: applyRun,
    loading: applyLoading,
  } = useRequest(queryGetContractList, {
    defaultParams: [{ created_by: user?.ID }],
    formatResult: contractResult,
  });
  //我的审批列表
  const {
    data: AuditData,
    run,
    loading: aduitLoading,
  } = useRequest((data) => queryGetContractList(data), {
    manual: true,
    formatResult: contractResult,
  });
  console.log('================', ApplyData, AuditData, OAData);

  //审核合同
  const { run: runCheck, loading: checkLoading } = useRequest(
    (data) => queryContractCheck(data),
    {
      manual: true,
      onSuccess: () => {
        setConVisible(false);
        message.success('审核成功');
      },
      onErroe: () => {
        message.error('审核失败');
      },
    },
  );

  useEffect(() => {
    if (user?.Permission['menu-001-audit']) run({ status: 1 });
  }, [user]);

  const onTabChange = (activeKey) => {
    applyRun();
    setTabActive(activeKey);
  };
  const columns = [
    {
      title: '标题',
      dataIndex: 'table_name',
    },
    {
      title: '摘要',
      dataIndex: 'table_desc',
      render: (descList) => {
        return (
          <ul>
            {descList?.map((item) => (
              <li>{item}</li>
            ))}
          </ul>
        );
      },
    },
    {
      title: '发起人',
      dataIndex: 'CName',
    },
    {
      title: '发起时间',
      render: (record) => {
        return dayjs(record.create_time).format('YYYY-MM-DD HH:mm:ss');
      },
    },
    {
      title: '流程状态',
      dataIndex: 'statusText',
    },
  ];
  const approveColumns = [
    {
      title: '标题',
      dataIndex: 'table_name',
    },
    {
      title: '摘要',
      dataIndex: 'table_desc',
      render: (descList) => {
        return (
          <ul>
            {descList?.map((item) => (
              <li>{item}</li>
            ))}
          </ul>
        );
      },
    },
    {
      title: '发起人',
      dataIndex: 'CName',
    },
    {
      title: '发起时间',
      render: (record) => {
        return dayjs(record.create_time).format('YYYY-MM-DD HH:mm:ss');
      },
    },
    {
      title: '流程状态',
      dataIndex: 'statusText',
    },
    {
      title: '操作',
      render: (record) => (
        <>
          <a
            style={{ color: '#4096ff' }}
            onClick={() => {
              if (record.type == TYPE.Contract) {
                setDetail(record);
                setConVisible(true);
              } else {
                navigate(`/oa/detail/${record.flow_id}/${record.id}`);
              }
            }}
          >
            审批
          </a>
        </>
      ),
    },
  ];
  const handleApplySubmit = (values) => {
    console.log(values);
  };
  const handleApproveSubmit = (values) => {
    console.log(values);
  };
  const renderPage = (activeKey) => {
    if (activeKey == '1')
      return (
        <>
          {' '}
          <Form
            name="basic"
            // labelCol={{ span: 0 }}
            // wrapperCol={{ span: 24 }}
            onFinish={handleApplySubmit}
            ref={applyFormRef}
          >
            <div style={{ display: 'flex' }}>
              <Form.Item name="range-picker" label="申请时间：">
                <RangePicker />
              </Form.Item>
              <Form.Item>
                <Button
                  type="primary"
                  htmlType="submit"
                  style={{ marginLeft: 10 }}
                >
                  查询
                </Button>
              </Form.Item>
            </div>
            <Table columns={columns} dataSource={ApplyData} />
          </Form>
        </>
      );
    else if (activeKey == '2')
      return (
        <>
          {' '}
          <Form
            name="basic"
            // labelCol={{ span: 0 }}
            // wrapperCol={{ span: 24 }}
            onFinish={handleApproveSubmit}
            ref={approveFormRef}
          >
            <div style={{ display: 'flex' }}>
              <Form.Item name="range-picker" label="审批时间：">
                <RangePicker />
              </Form.Item>
              <Form.Item>
                <Button
                  type="primary"
                  htmlType="submit"
                  style={{ marginLeft: 10 }}
                >
                  查询
                </Button>
              </Form.Item>
            </div>
          </Form>
          <Table
            rowKey="id"
            columns={approveColumns}
            dataSource={[...OAData, ...AuditData].sort(
              (a, b) => a.create_time > b.create_time,
            )}
          />
        </>
      );
  };
  return (
    <PageContainer
      header={{
        title: '个人中心',
      }}
      tabList={[
        {
          tab: '我的申请',
          key: '1',
        },
        {
          tab: '我的审批',
          key: '2',
        },
      ]}
      onTabChange={onTabChange}
    >
      <div>{renderPage(tabActive)}</div>
      <ContractModal
        detail={detail}
        type={Type.check}
        // projectList={projectData?.list}
        visible={conVisible}
        handleOk={(data) =>
          detail.status == Status.Checking ? runCheck(data) : null
        }
        handleCancel={() => setConVisible(false)}
      />
    </PageContainer>
  );
}
export default profile;
